﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using SIX.TimApi;

namespace BankomatPaymentEcr
{


    /**
     * <p>Example ECR implementation using TimApi.</p>
     * 
     * <p>Uses asynchronous handling of terminal events to provide a simple ECR
     * allowing to do purchase, balance and reversal of the last purchase.</p>
     */

    public partial class BankomatForm : Form
    {
        int COMMAND_TIMEOUT = 60000;

        BankomatPaymentProvider paymentProvider { get; set; }
        public decimal amount { get; set; }
        public string currency { get; set; }

        Terminal terminal;
        string reversableTrxRefNum = "";
        bool requestInProgress = false;

        public BankomatForm()
        {
            InitializeComponent();

            TerminalSettings settings = new TerminalSettings();
            terminal = new SIX.TimApi.Terminal(settings);

            /**
	         * <p>Listen to terminal events updating the UI in the main thread.</p>
	         * 
	         * <p>This example implementation handles all functions supported by the Example ECR.
	         * You can use multiple event handlers to handle individual events differently if required.
	         * Handlers can be added and removed any time.</p>
	         * 
	         * <p> For this example only
	         * {@link TerminalListener#transactionCompleted()} and {@link TerminalListener#balanceCompleted()}
	         * is used to enable the buttons again when the terminal is ready for another transaction.</p> 
	         */
            terminal.TerminalStatusChanged += new Terminal.TerminalStatusChangedHandler(terminal_TerminalStatusChanged);
            terminal.TransactionCompleted += new Terminal.TransactionCompletedEventHandler(terminal_TransactionCompleted);
            terminal.BalanceCompleted += new Terminal.BalanceCompletedEventHandler(terminal_BalanceCompleted);

            terminal.PosId = "Kassa1";
            this.timerCommandTimeout.Tick += timerCommandTimeout_Tick;
        }
        #region Terminal Event Handlers
        void terminal_TerminalStatusChanged(object sender, SIX.TimApi.TerminalStatus trmStatus)
        {
            lbDisplay.Text = "";

            foreach (string s in trmStatus.DisplayContent)
            {
                lbDisplay.Text += s + "\r\n";
            }
        }
        void terminal_TransactionCompleted(object sender, SIX.TimApi.Terminal.TransactionCompletedEventArgs transactionCompletedEventArgs)
        {
            setRequestInProgress(false);

            // If event contains a null exception the transaction completed successfully.
            // Use data.getTransactionType() to see what kind of transaction finished if you
            // do not track this information yourself already. getTransactionType() is
            // present for your convenience.
            if (transactionCompletedEventArgs.TimError == null)
            {
                if (transactionCompletedEventArgs.TransactionResponse.TransactionType == SIX.TimApi.Constants.TransactionType.Purchase)
                {
                    setReversableTransaction(transactionCompletedEventArgs.TransactionResponse.TransactionInformation.TransSeq);
                }
                else
                {
                    setReversableTransaction("");
                }

                if (paymentProvider != null)
                {
                    paymentProvider.OnPurchaseOK(this.amount);
                }
            }
            else
            {
                // If event contains an error the transaction failed. Show an error message.
                // The exception contains the error code and additional information if present.
                // The error message is provided in the exception.
                //MessageBox.Show(transactionCompletedEventArgs.TimError.ErrorMessage, "Transaction failed");
                IntraSell_DLL.ModuleLog.WriteLog("Transaction failed: " + transactionCompletedEventArgs.TimError.ErrorMessage);

                if (paymentProvider != null)
                {
                    paymentProvider.OnPurchaseFailed("Transaction failed: " + transactionCompletedEventArgs.TimError.ErrorMessage);
                }
            }
        }
        void terminal_BalanceCompleted(object sender, Terminal.BalanceCompletedEventArgs balanceCompletedEventArgs)
        {
            //logger.info("Balance completed");
            setRequestInProgress(false);

            if (balanceCompletedEventArgs.Error != null)
            {
                MessageBox.Show(balanceCompletedEventArgs.TimError.ErrorMessage, "Balance failed");
            }
            else
            {
                BalanceResponse balance = balanceCompletedEventArgs.BalanceResponse;
                string merchantReceipt = balance.PrintData.MerchantReceipt;
                string cR = balance.PrintData.CardholderReceipt;

                if (paymentProvider != null)
                {
                    paymentProvider.OnBalanceOK(merchantReceipt);
                }
            }
        }
        #endregion
        /**
	     * Set transaction reference number of last transaction that can be reversed.
	     * Set to null after reversal.
	     */
        void setReversableTransaction(string transactionReferenceNumber)
        {
            reversableTrxRefNum = transactionReferenceNumber;
            btnReversal.Enabled = (!requestInProgress && transactionReferenceNumber != "");
        }
        /**
	    * Do a purchase transaction.
	    */
        public void onPurchase(BankomatPaymentProvider paymentProvider, decimal amount, string currency)
        {
            try
            {
                this.amount = amount;
                this.currency = currency;
                this.tbAmount.Text = amount.ToString("F2");

                this.paymentProvider = paymentProvider;
                //logger.info("Begin purchase");
                setRequestInProgress(true);
                // Clear the transaction data. It is used for reversal but not purchase
                terminal.TransactionData = null;
                // Run the transaction. Once completed the event transactionCompleted is raised.
                terminal.TransactionAsync(SIX.TimApi.Constants.TransactionType.Purchase, new SIX.TimApi.Amount(amount, currency));

                this.timerCommandTimeout.Interval = COMMAND_TIMEOUT;
                this.timerCommandTimeout.Enabled = true;
                this.timerCommandTimeout.Start();
            }
            catch (SIX.TimApi.TimException ex)
            {
                setRequestInProgress(false);
                //MessageBox.Show(exc.ErrorMessage, "Transaction failed");
                IntraSell_DLL.ModuleLog.WriteLog("onPurchase failed: " + ex.ErrorMessage);
                throw ex;
            }
        }

        private void timerCommandTimeout_Tick(object sender, EventArgs e)
        {
            if (paymentProvider != null)
            {
                paymentProvider.OnPurchaseFailed("Transaction failed: Timeout");
            }
            this.timerCommandTimeout.Enabled = false;
            this.timerCommandTimeout.Stop();
        }
        /**
	     * Cancel running transaction.
	     */
        private void onCancel()
        {
            try
            {
                //logger.info("Cancel");
                terminal.Cancel();
            }
            catch (TimException ex)
            {
                // MessageBox.Show(e.ErrorMessage, "Cancel failed");
                IntraSell_DLL.ModuleLog.WriteLog("onCancel failed: " + ex.ErrorMessage);
                throw ex;
            }
        }
        /**
	     * Do a reversal transaction.
	     */
        private void onReversal()
        {
            try
            {
                //logger.info("Begin reversal");
                setRequestInProgress(true);

                // Set the transaction data. Reversal needs the transaction reference number
                // of the previous transaction
                TransactionData trxData = new TransactionData();
                trxData.TransSeq = reversableTrxRefNum;
                terminal.TransactionData = trxData;

                // Run the transaction. Once completed the listener receives a
                // transactionCompleted notification
                terminal.TransactionAsync(SIX.TimApi.Constants.TransactionType.Reversal, new Amount(System.Convert.ToDecimal(tbAmount.Text), "CHF"));

            }
            catch (TimException ex)
            {
                setRequestInProgress(false);
                //MessageBox.Show(e.ErrorMessage, "Transaction failed");
                IntraSell_DLL.ModuleLog.WriteLog("onReversal failed: " + ex.ErrorMessage);
                throw ex;
            }
        }
        /**
	     * Do a balance.
	     */
        public void onBalance(BankomatPaymentProvider paymentProvider)
        {
            try
            {
                this.paymentProvider = paymentProvider;
                setRequestInProgress(true);
                terminal.BalanceAsync();
            }
            catch (TimException ex)
            {
                setRequestInProgress(false);
                //MessageBox.Show(e.ErrorMessage, "Transaction failed");
                IntraSell_DLL.ModuleLog.WriteLog("onBalance failed: " + ex.ErrorMessage);
                throw ex;
            }
        }
        /**
	     * Set if a transaction or balance request is running. If true purchase, balance
	     * and reversal buttons are disabled and cancel button enabled.
	     */
        private void setRequestInProgress(bool inProgress)
        {
            requestInProgress = inProgress;

            btnPurchase.Enabled = !inProgress;
            btnReversal.Enabled = (!inProgress && reversableTrxRefNum != "");
            btnCancel.Enabled = inProgress;
            btnBalance.Enabled = !inProgress;

            if (!inProgress)
            {
                timerCommandTimeout.Stop();
            }
        }
        # region ButtonEvents
        private void btnPurchase_Click(object sender, EventArgs e)
        {
            onPurchase(null, System.Convert.ToDecimal(tbAmount.Text), tbCurrency.Text);
        }
        private void btnCancel_Click(object sender, EventArgs e)
        {
            onCancel();
        }
        private void btnBalance_Click(object sender, EventArgs e)
        {
            onBalance(null);
        }
        private void btnReversal_Click(object sender, EventArgs e)
        {
            onReversal();
        }
        #endregion


        public void DisposeTerminal()
        {
            try
            {
                terminal.Dispose();
                Terminal.TimApiDispose();
            }
            catch (SIX.TimApi.TimException ex)
            {
                setRequestInProgress(false);
                //MessageBox.Show(exc.ErrorMessage, "Transaction failed");
                IntraSell_DLL.ModuleLog.WriteLog("Dispose failed: " + ex.ErrorMessage);
                throw ex;
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            DisposeTerminal();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            //Copy TimApi config because intrasell is installed in unknow directory 
            if (!System.IO.File.Exists("c:\\IntraSell_Net\\TimApi.cfg"))
                System.IO.File.Copy("TimApi.cfg", "c:\\IntraSell_Net\\TimApi.cfg");
            if (System.IO.File.Exists("c:\\IntraSell_Net\\TimApi.cfg"))
            {
                System.IO.File.Delete("TimApi.cfg");
                System.IO.File.Copy("c:\\IntraSell_Net\\TimApi.cfg", "TimApi.cfg");
            }
        }

        #region "Test"

        /// <summary>
        /// For tests send PurchaseOK to IntraSell
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPurchaseOK_Click(object sender, EventArgs e)
        {
            if (paymentProvider != null)
            {
                paymentProvider.OnPurchaseOK(-1 * this.amount);
            }
        }

        #endregion



    }
}
